﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Data;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using Telerik.Windows.Controls;

namespace RadButtonBug
{
	public class ButtonExtensions
	{
		private static Dictionary<FrameworkElement, Helper> helpers = new Dictionary<FrameworkElement, Helper>();

		public static readonly DependencyProperty CommandToEnsureProperty =
			DependencyProperty.RegisterAttached("CommandToEnsure", typeof(object), typeof(ButtonExtensions), new PropertyMetadata(false, OnCommandToEnsureChanged));

		public static object GetCommandToEnsure(DependencyObject obj)
		{
			return (object)obj.GetValue(CommandToEnsureProperty);
		}

		public static void SetCommandToEnsure(DependencyObject obj, object value)
		{
			obj.SetValue(CommandToEnsureProperty, value);
		}

		private static void OnCommandToEnsureChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var element = d as FrameworkElement;
			if (element != null)
			{
				if (e.NewValue != null)
				{
					Binding binding = new Binding("IsEnabled");
					binding.Source = element;

					Helper newHelper = new Helper();
					newHelper.Command = e.NewValue as DelegateCommand;
					newHelper.SetBinding(Helper.IsEnabledHelperProperty, binding);

					helpers[element] = newHelper;
				}
				else
				{
					helpers.Remove(element);
				}
			}
		}
	}

	public class Helper : FrameworkElement
	{
		public static readonly DependencyProperty IsEnabledHelperProperty =
			DependencyProperty.Register("IsEnabledHelper", typeof(bool), typeof(Helper), new PropertyMetadata(true, OnIsEnabledHelperPropertyChanged));

		public bool IsEnabledHelper
		{
			get { return (bool)GetValue(IsEnabledHelperProperty); }
			set { SetValue(IsEnabledHelperProperty, value); }
		}

		public DelegateCommand Command { get; set; }

		private static void OnIsEnabledHelperPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var helper = d as Helper;
			if (helper != null && helper.Command != null)
			{
				if ((bool)e.NewValue)
					helper.Command.InvalidateCanExecute();
			}
		}
	}
}
